@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-7 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 2000-2007 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ int mutt_account_match (const ACCOUNT* a1, const ACCOUNT* a2)
     return 0;
 
 #ifdef USE_IMAP
-  if (a1->type == M_ACCT_TYPE_IMAP)
+  if (a1->type == MUTT_ACCT_TYPE_IMAP)
   {
     if (ImapUser)
       user = ImapUser;
@@ -47,15 +47,15 @@ int mutt_account_match (const ACCOUNT* a1, const ACCOUNT* a2)
 #endif
 
 #ifdef USE_POP
-  if (a1->type == M_ACCT_TYPE_POP && PopUser)
+  if (a1->type == MUTT_ACCT_TYPE_POP && PopUser)
     user = PopUser;
 #endif
   
-  if (a1->flags & a2->flags & M_ACCT_USER)
+  if (a1->flags & a2->flags & MUTT_ACCT_USER)
     return (!strcmp (a1->user, a2->user));
-  if (a1->flags & M_ACCT_USER)
+  if (a1->flags & MUTT_ACCT_USER)
     return (!strcmp (a1->user, user));
-  if (a2->flags & M_ACCT_USER)
+  if (a2->flags & MUTT_ACCT_USER)
     return (!strcmp (a2->user, user));
 
   return 1;
@@ -73,17 +73,17 @@ int mutt_account_fromurl (ACCOUNT* account, ciss_url_t* url)
   if (url->user)
   {
     strfcpy (account->user, url->user, sizeof (account->user));
-    account->flags |= M_ACCT_USER;
+    account->flags |= MUTT_ACCT_USER;
   }
   if (url->pass)
   {
     strfcpy (account->pass, url->pass, sizeof (account->pass));
-    account->flags |= M_ACCT_PASS;
+    account->flags |= MUTT_ACCT_PASS;
   }
   if (url->port)
   {
     account->port = url->port;
-    account->flags |= M_ACCT_PORT;
+    account->flags |= MUTT_ACCT_PORT;
   }
 
   return 0;
@@ -101,9 +101,9 @@ void mutt_account_tourl (ACCOUNT* account, ciss_url_t* url)
   url->port = 0;
 
 #ifdef USE_IMAP
-  if (account->type == M_ACCT_TYPE_IMAP)
+  if (account->type == MUTT_ACCT_TYPE_IMAP)
   {
-    if (account->flags & M_ACCT_SSL)
+    if (account->flags & MUTT_ACCT_SSL)
       url->scheme = U_IMAPS;
     else
       url->scheme = U_IMAP;
@@ -111,9 +111,9 @@ void mutt_account_tourl (ACCOUNT* account, ciss_url_t* url)
 #endif
 
 #ifdef USE_POP
-  if (account->type == M_ACCT_TYPE_POP)
+  if (account->type == MUTT_ACCT_TYPE_POP)
   {
-    if (account->flags & M_ACCT_SSL)
+    if (account->flags & MUTT_ACCT_SSL)
       url->scheme = U_POPS;
     else
       url->scheme = U_POP;
@@ -121,9 +121,9 @@ void mutt_account_tourl (ACCOUNT* account, ciss_url_t* url)
 #endif
 
 #ifdef USE_SMTP
-  if (account->type == M_ACCT_TYPE_SMTP)
+  if (account->type == MUTT_ACCT_TYPE_SMTP)
   {
-    if (account->flags & M_ACCT_SSL)
+    if (account->flags & MUTT_ACCT_SSL)
       url->scheme = U_SMTPS;
     else
       url->scheme = U_SMTP;
@@ -131,11 +131,11 @@ void mutt_account_tourl (ACCOUNT* account, ciss_url_t* url)
 #endif
 
   url->host = account->host;
-  if (account->flags & M_ACCT_PORT)
+  if (account->flags & MUTT_ACCT_PORT)
     url->port = account->port;
-  if (account->flags & M_ACCT_USER)
+  if (account->flags & MUTT_ACCT_USER)
     url->user = account->user;
-  if (account->flags & M_ACCT_PASS)
+  if (account->flags & MUTT_ACCT_PASS)
     url->pass = account->pass;
 }
 
@@ -145,14 +145,14 @@ int mutt_account_getuser (ACCOUNT* account)
   char prompt[SHORT_STRING];
 
   /* already set */
-  if (account->flags & M_ACCT_USER)
+  if (account->flags & MUTT_ACCT_USER)
     return 0;
 #ifdef USE_IMAP
-  else if ((account->type == M_ACCT_TYPE_IMAP) && ImapUser)
+  else if ((account->type == MUTT_ACCT_TYPE_IMAP) && ImapUser)
     strfcpy (account->user, ImapUser, sizeof (account->user));
 #endif
 #ifdef USE_POP
-  else if ((account->type == M_ACCT_TYPE_POP) && PopUser)
+  else if ((account->type == MUTT_ACCT_TYPE_POP) && PopUser)
     strfcpy (account->user, PopUser, sizeof (account->user));
 #endif
   else if (option (OPTNOCURSES))
@@ -166,7 +166,7 @@ int mutt_account_getuser (ACCOUNT* account)
       return -1;
   }
 
-  account->flags |= M_ACCT_USER;
+  account->flags |= MUTT_ACCT_USER;
 
   return 0;
 }
@@ -174,26 +174,26 @@ int mutt_account_getuser (ACCOUNT* account)
 int mutt_account_getlogin (ACCOUNT* account)
 {
   /* already set */
-  if (account->flags & M_ACCT_LOGIN)
+  if (account->flags & MUTT_ACCT_LOGIN)
     return 0;
 #ifdef USE_IMAP
-  else if (account->type == M_ACCT_TYPE_IMAP)
+  else if (account->type == MUTT_ACCT_TYPE_IMAP)
   {
     if (ImapLogin)
     {
       strfcpy (account->login, ImapLogin, sizeof (account->login));
-      account->flags |= M_ACCT_LOGIN;
+      account->flags |= MUTT_ACCT_LOGIN;
     }
   }
 #endif
 
-  if (!(account->flags & M_ACCT_LOGIN))
+  if (!(account->flags & MUTT_ACCT_LOGIN))
   {
     mutt_account_getuser (account);
     strfcpy (account->login, account->user, sizeof (account->login));
   }
 
-  account->flags |= M_ACCT_LOGIN;
+  account->flags |= MUTT_ACCT_LOGIN;
 
   return 0;
 }
@@ -203,18 +203,18 @@ int mutt_account_getpass (ACCOUNT* account)
 {
   char prompt[SHORT_STRING];
 
-  if (account->flags & M_ACCT_PASS)
+  if (account->flags & MUTT_ACCT_PASS)
     return 0;
 #ifdef USE_IMAP
-  else if ((account->type == M_ACCT_TYPE_IMAP) && ImapPass)
+  else if ((account->type == MUTT_ACCT_TYPE_IMAP) && ImapPass)
     strfcpy (account->pass, ImapPass, sizeof (account->pass));
 #endif
 #ifdef USE_POP
-  else if ((account->type == M_ACCT_TYPE_POP) && PopPass)
+  else if ((account->type == MUTT_ACCT_TYPE_POP) && PopPass)
     strfcpy (account->pass, PopPass, sizeof (account->pass));
 #endif
 #ifdef USE_SMTP
-  else if ((account->type == M_ACCT_TYPE_SMTP) && SmtpPass)
+  else if ((account->type == MUTT_ACCT_TYPE_SMTP) && SmtpPass)
     strfcpy (account->pass, SmtpPass, sizeof (account->pass));
 #endif
   else if (option (OPTNOCURSES))
@@ -222,19 +222,104 @@ int mutt_account_getpass (ACCOUNT* account)
   else
   {
     snprintf (prompt, sizeof (prompt), _("Password for %s@%s: "),
-              account->flags & M_ACCT_LOGIN ? account->login : account->user,
+              account->flags & MUTT_ACCT_LOGIN ? account->login : account->user,
               account->host);
     account->pass[0] = '\0';
     if (mutt_get_password (prompt, account->pass, sizeof (account->pass)))
       return -1;
   }
 
-  account->flags |= M_ACCT_PASS;
+  account->flags |= MUTT_ACCT_PASS;
 
   return 0;
 }
 
 void mutt_account_unsetpass (ACCOUNT* account)
 {
-  account->flags &= ~M_ACCT_PASS;
+  account->flags &= ~MUTT_ACCT_PASS;
+}
+
+/* mutt_account_getoauthbearer: call external command to generate the
+ * oauth refresh token for this ACCOUNT, then create and encode the
+ * OAUTHBEARER token based on RFC 7628.  Returns NULL on failure.
+ * Resulting token is dynamically allocated and should be FREE'd by the
+ * caller.
+ */
+char* mutt_account_getoauthbearer (ACCOUNT* account)
+{
+  FILE	*fp;
+  char *cmd = NULL;
+  char *token = NULL;
+  size_t token_size = 0;
+  char *oauthbearer = NULL;
+  size_t oalen;
+  char *encoded_token = NULL;
+  size_t encoded_len;
+  pid_t	pid;
+
+  /* The oauthbearer token includes the login */
+  if (mutt_account_getlogin (account))
+    return NULL;
+
+#ifdef USE_IMAP
+  if ((account->type == MUTT_ACCT_TYPE_IMAP) && ImapOauthRefreshCmd)
+    cmd = ImapOauthRefreshCmd;
+#endif
+#ifdef USE_POP
+  else if ((account->type == MUTT_ACCT_TYPE_POP) && PopOauthRefreshCmd)
+    cmd = PopOauthRefreshCmd;
+#endif
+#ifdef USE_SMTP
+  else if ((account->type == MUTT_ACCT_TYPE_SMTP) && SmtpOauthRefreshCmd)
+    cmd = SmtpOauthRefreshCmd;
+#endif
+
+  if (cmd == NULL)
+  {
+    /* L10N: You will see this error message if (1) you have "oauthbearer" in
+       one of your $*_authenticators and (2) you do not have the corresponding
+       $*_oauth_refresh_command defined. So the message does not mean "None of
+       your $*_oauth_refresh_command's are defined."
+     */
+    mutt_error (_("mutt_account_getoauthbearer: No OAUTH refresh command defined"));
+    return NULL;
+  }
+
+  if ((pid = mutt_create_filter (cmd, NULL, &fp, NULL)) < 0)
+  {
+    mutt_perror _("mutt_account_getoauthbearer: Unable to run refresh command");
+    return NULL;
+  }
+
+  /* read line */
+  token = mutt_read_line (NULL, &token_size, fp, NULL, 0);
+  safe_fclose (&fp);
+  mutt_wait_filter (pid);
+
+  if (token == NULL || *token == '\0')
+  {
+    mutt_error (_("mutt_account_getoauthbearer: Command returned empty string"));
+    FREE (&token);
+    return NULL;
+  }
+
+  /* Determine the length of the keyed message digest, add 50 for
+   * overhead.
+   */
+  oalen = strlen (account->login) + strlen (account->host) + strlen (token) + 50; 
+  oauthbearer = safe_malloc (oalen);
+
+  snprintf (oauthbearer, oalen,
+    "n,a=%s,\001host=%s\001port=%d\001auth=Bearer %s\001\001",
+    account->login, account->host, account->port, token);
+
+  FREE (&token);
+
+  encoded_len = strlen (oauthbearer) * 4 / 3 + 10;
+  encoded_token = safe_malloc (encoded_len);
+  mutt_to_base64 ((unsigned char*) encoded_token,
+                  (unsigned char*) oauthbearer, strlen (oauthbearer),
+		  encoded_len);
+  FREE (&oauthbearer);
+  return encoded_token;
 }
